# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/18a_callback.training.ipynb (unless otherwise specified).

__all__ = ['ShortEpochCallback', 'GradientAccumulation', 'set_bn_eval', 'BnFreeze', 'bn_types']

# Cell
from ..basics import *
from .progress import *
from .fp16 import *

# Cell
@log_args
class ShortEpochCallback(Callback):
    "Fit just `pct` of an epoch, then stop"
    def __init__(self,pct=0.01,short_valid=True): self.pct,self.short_valid = pct,short_valid
    def after_batch(self):
        if self.iter/self.n_iter < self.pct: return
        if self.training:    raise CancelTrainException
        if self.short_valid: raise CancelValidException

# Cell
@log_args
class GradientAccumulation(Callback):
    "Accumulate gradients before updating weights"
    toward_end,run_before=True,MixedPrecision

    def __init__(self, n_acc=32): store_attr('n_acc')
    def before_fit(self): self.count=0

    def after_backward(self):
        self.count += find_bs(self.learn.yb)
        if self.count < self.n_acc: raise CancelBatchException() #skip weight update
        else: self.count=0

    _docs = dict(before_fit="Set counter to 0",
                 after_backward="Skip weight update if we have not seen enough items")

# Cell
bn_types = (nn.BatchNorm1d, nn.BatchNorm2d, nn.BatchNorm3d)

def set_bn_eval(m:nn.Module, use_eval=True)->None:
    "Set bn layers in eval mode for all recursive children of `m`."
    for l in m.children():
        if isinstance(l, bn_types) and not next(l.parameters()).requires_grad:
            if use_eval: l.eval()
            else:        l.train()
        set_bn_eval(l)

class BnFreeze(Callback):
    "Freeze moving average statistics in all non-trainable batchnorm layers."
    def before_epoch(self):
        set_bn_eval(self.model)


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"subplots": "00_torch_core.ipynb",
         "show_image": "00_torch_core.ipynb",
         "show_titled_image": "00_torch_core.ipynb",
         "show_images": "00_torch_core.ipynb",
         "ArrayBase": "00_torch_core.ipynb",
         "ArrayImageBase": "00_torch_core.ipynb",
         "ArrayImage": "00_torch_core.ipynb",
         "ArrayImageBW": "00_torch_core.ipynb",
         "ArrayMask": "00_torch_core.ipynb",
         "Tensor.__array_eq__": "00_torch_core.ipynb",
         "tensor": "00_torch_core.ipynb",
         "set_seed": "00_torch_core.ipynb",
         "unsqueeze": "00_torch_core.ipynb",
         "unsqueeze_": "00_torch_core.ipynb",
         "apply": "00_torch_core.ipynb",
         "maybe_gather": "00_torch_core.ipynb",
         "to_detach": "00_torch_core.ipynb",
         "to_half": "00_torch_core.ipynb",
         "to_float": "00_torch_core.ipynb",
         "defaults.use_cuda": "00_torch_core.ipynb",
         "default_device": "00_torch_core.ipynb",
         "to_device": "00_torch_core.ipynb",
         "to_cpu": "00_torch_core.ipynb",
         "to_np": "00_torch_core.ipynb",
         "to_concat": "00_torch_core.ipynb",
         "Tensor.set_meta": "00_torch_core.ipynb",
         "Tensor.get_meta": "00_torch_core.ipynb",
         "Tensor.as_subclass": "00_torch_core.ipynb",
         "TensorBase": "00_torch_core.ipynb",
         "TensorCategory": "00_torch_core.ipynb",
         "TensorMultiCategory": "00_torch_core.ipynb",
         "TensorImageBase": "00_torch_core.ipynb",
         "TensorImage": "00_torch_core.ipynb",
         "TensorImageBW": "00_torch_core.ipynb",
         "TensorMask": "00_torch_core.ipynb",
         "TitledTensorScalar": "00_torch_core.ipynb",
         "L.tensored": "00_torch_core.ipynb",
         "L.stack": "00_torch_core.ipynb",
         "L.cat": "00_torch_core.ipynb",
         "concat": "00_torch_core.ipynb",
         "Chunks": "00_torch_core.ipynb",
         "show_title": "00_torch_core.ipynb",
         "ShowTitle": "00_torch_core.ipynb",
         "TitledInt": "00_torch_core.ipynb",
         "TitledFloat": "00_torch_core.ipynb",
         "TitledStr": "00_torch_core.ipynb",
         "TitledTuple": "00_torch_core.ipynb",
         "TitledStr.truncate": "00_torch_core.ipynb",
         "pd.DataFrame.__init__": "00_torch_core.ipynb",
         "get_empty_df": "00_torch_core.ipynb",
         "display_df": "00_torch_core.ipynb",
         "get_first": "00_torch_core.ipynb",
         "one_param": "00_torch_core.ipynb",
         "item_find": "00_torch_core.ipynb",
         "find_device": "00_torch_core.ipynb",
         "find_bs": "00_torch_core.ipynb",
         "np_func": "00_torch_core.ipynb",
         "Module": "00_torch_core.ipynb",
         "get_model": "00_torch_core.ipynb",
         "one_hot": "00_torch_core.ipynb",
         "one_hot_decode": "00_torch_core.ipynb",
         "params": "00_torch_core.ipynb",
         "trainable_params": "00_torch_core.ipynb",
         "norm_types": "00_torch_core.ipynb",
         "norm_bias_params": "00_torch_core.ipynb",
         "batch_to_samples": "00_torch_core.ipynb",
         "Tensor.interp_1d": "00_torch_core.ipynb",
         "Tensor.pca": "00_torch_core.ipynb",
         "logit": "00_torch_core.ipynb",
         "num_distrib": "00_torch_core.ipynb",
         "rank_distrib": "00_torch_core.ipynb",
         "distrib_barrier": "00_torch_core.ipynb",
         "Path.save_array": "00_torch_core.ipynb",
         "Path.load_array": "00_torch_core.ipynb",
         "base_doc": "00_torch_core.ipynb",
         "doc": "00_torch_core.ipynb",
         "nested_reorder": "00_torch_core.ipynb",
         "make_cross_image": "00_torch_core.ipynb",
         "show_image_batch": "00_torch_core.ipynb",
         "requires_grad": "00_torch_core.ipynb",
         "init_default": "01_layers.ipynb",
         "cond_init": "00_torch_core.ipynb",
         "apply_leaf": "00_torch_core.ipynb",
         "apply_init": "00_torch_core.ipynb",
         "script_use_ctx": "00_torch_core.ipynb",
         "script_save_ctx": "00_torch_core.ipynb",
         "script_fwd": "00_torch_core.ipynb",
         "script_bwd": "00_torch_core.ipynb",
         "grad_module": "00_torch_core.ipynb",
         "module": "01_layers.ipynb",
         "Identity": "01_layers.ipynb",
         "Lambda": "01_layers.ipynb",
         "PartialLambda": "01_layers.ipynb",
         "Flatten": "01_layers.ipynb",
         "View": "01_layers.ipynb",
         "ResizeBatch": "01_layers.ipynb",
         "Debugger": "01_layers.ipynb",
         "sigmoid_range": "01_layers.ipynb",
         "SigmoidRange": "01_layers.ipynb",
         "AdaptiveConcatPool1d": "01_layers.ipynb",
         "AdaptiveConcatPool2d": "01_layers.ipynb",
         "PoolType": "01_layers.ipynb",
         "adaptive_pool": "01_layers.ipynb",
         "PoolFlatten": "01_layers.ipynb",
         "NormType": "01_layers.ipynb",
         "BatchNorm": "01_layers.ipynb",
         "InstanceNorm": "01_layers.ipynb",
         "BatchNorm1dFlat": "01_layers.ipynb",
         "LinBnDrop": "01_layers.ipynb",
         "sigmoid": "01_layers.ipynb",
         "sigmoid_": "01_layers.ipynb",
         "vleaky_relu": "01_layers.ipynb",
         "init_linear": "01_layers.ipynb",
         "defaults.activation": "01_layers.ipynb",
         "ConvLayer": "01_layers.ipynb",
         "AdaptiveAvgPool": "01_layers.ipynb",
         "MaxPool": "01_layers.ipynb",
         "AvgPool": "01_layers.ipynb",
         "trunc_normal_": "01_layers.ipynb",
         "Embedding": "01_layers.ipynb",
         "SelfAttention": "01_layers.ipynb",
         "PooledSelfAttention2d": "01_layers.ipynb",
         "SimpleSelfAttention": "01_layers.ipynb",
         "icnr_init": "01_layers.ipynb",
         "PixelShuffle_ICNR": "01_layers.ipynb",
         "sequential": "01_layers.ipynb",
         "SequentialEx": "01_layers.ipynb",
         "MergeLayer": "01_layers.ipynb",
         "Cat": "01_layers.ipynb",
         "SimpleCNN": "01_layers.ipynb",
         "ProdLayer": "01_layers.ipynb",
         "inplace_relu": "01_layers.ipynb",
         "SEModule": "01_layers.ipynb",
         "ResBlock": "01_layers.ipynb",
         "SEBlock": "01_layers.ipynb",
         "SEResNeXtBlock": "01_layers.ipynb",
         "SeparableBlock": "01_layers.ipynb",
         "swish": "01_layers.ipynb",
         "Swish": "01_layers.ipynb",
         "MishJitAutoFn": "01_layers.ipynb",
         "mish": "01_layers.ipynb",
         "Mish": "01_layers.ipynb",
         "ParameterModule": "01_layers.ipynb",
         "children_and_parameters": "01_layers.ipynb",
         "nn.Module.has_children": "01_layers.ipynb",
         "flatten_model": "01_layers.ipynb",
         "NoneReduce": "01_layers.ipynb",
         "in_channels": "01_layers.ipynb",
         "BaseLoss": "01a_losses.ipynb",
         "CrossEntropyLossFlat": "01a_losses.ipynb",
         "BCEWithLogitsLossFlat": "01a_losses.ipynb",
         "BCELossFlat": "01a_losses.ipynb",
         "MSELossFlat": "01a_losses.ipynb",
         "L1LossFlat": "01a_losses.ipynb",
         "LabelSmoothingCrossEntropy": "01a_losses.ipynb",
         "LabelSmoothingCrossEntropyFlat": "01a_losses.ipynb",
         "fa_collate": "02_data.load.ipynb",
         "fa_convert": "02_data.load.ipynb",
         "SkipItemException": "02_data.load.ipynb",
         "DataLoader": "02_data.load.ipynb",
         "TfmdDL": "03_data.core.ipynb",
         "DataLoaders": "03_data.core.ipynb",
         "FilteredBase": "03_data.core.ipynb",
         "TfmdLists": "03_data.core.ipynb",
         "decode_at": "03_data.core.ipynb",
         "show_at": "03_data.core.ipynb",
         "Datasets": "03_data.core.ipynb",
         "test_set": "03_data.core.ipynb",
         "DataLoaders.test_dl": "03_data.core.ipynb",
         "Config": "04_data.external.ipynb",
         "URLs": "04_data.external.ipynb",
         "download_url": "04_data.external.ipynb",
         "download_data": "04_data.external.ipynb",
         "file_extract": "04_data.external.ipynb",
         "newest_folder": "04_data.external.ipynb",
         "rename_extracted": "04_data.external.ipynb",
         "untar_data": "04_data.external.ipynb",
         "get_files": "05_data.transforms.ipynb",
         "FileGetter": "05_data.transforms.ipynb",
         "image_extensions": "05_data.transforms.ipynb",
         "get_image_files": "05_data.transforms.ipynb",
         "ImageGetter": "05_data.transforms.ipynb",
         "get_text_files": "05_data.transforms.ipynb",
         "ItemGetter": "05_data.transforms.ipynb",
         "AttrGetter": "05_data.transforms.ipynb",
         "RandomSplitter": "05_data.transforms.ipynb",
         "TrainTestSplitter": "05_data.transforms.ipynb",
         "IndexSplitter": "05_data.transforms.ipynb",
         "GrandparentSplitter": "05_data.transforms.ipynb",
         "FuncSplitter": "05_data.transforms.ipynb",
         "MaskSplitter": "05_data.transforms.ipynb",
         "FileSplitter": "05_data.transforms.ipynb",
         "ColSplitter": "05_data.transforms.ipynb",
         "RandomSubsetSplitter": "05_data.transforms.ipynb",
         "parent_label": "05_data.transforms.ipynb",
         "RegexLabeller": "05_data.transforms.ipynb",
         "ColReader": "05_data.transforms.ipynb",
         "CategoryMap": "05_data.transforms.ipynb",
         "Categorize": "05_data.transforms.ipynb",
         "Category": "05_data.transforms.ipynb",
         "MultiCategorize": "05_data.transforms.ipynb",
         "MultiCategory": "05_data.transforms.ipynb",
         "OneHotEncode": "05_data.transforms.ipynb",
         "EncodedMultiCategorize": "05_data.transforms.ipynb",
         "RegressionSetup": "05_data.transforms.ipynb",
         "get_c": "05_data.transforms.ipynb",
         "ToTensor": "05_data.transforms.ipynb",
         "IntToFloatTensor": "05_data.transforms.ipynb",
         "broadcast_vec": "05_data.transforms.ipynb",
         "Normalize": "05_data.transforms.ipynb",
         "TransformBlock": "06_data.block.ipynb",
         "CategoryBlock": "06_data.block.ipynb",
         "MultiCategoryBlock": "06_data.block.ipynb",
         "RegressionBlock": "06_data.block.ipynb",
         "DataBlock": "06_data.block.ipynb",
         "DataBlock.summary": "06_data.block.ipynb",
         "imagenet_stats": "07_vision.core.ipynb",
         "cifar_stats": "07_vision.core.ipynb",
         "mnist_stats": "07_vision.core.ipynb",
         "n_px": "07_vision.core.ipynb",
         "shape": "60_medical.imaging.ipynb",
         "aspect": "07_vision.core.ipynb",
         "Image.Image.reshape": "07_vision.core.ipynb",
         "Image.Image.to_bytes_format": "07_vision.core.ipynb",
         "Image.Image.to_thumb": "07_vision.core.ipynb",
         "Image.Image.resize_max": "07_vision.core.ipynb",
         "to_image": "07_vision.core.ipynb",
         "load_image": "07_vision.core.ipynb",
         "image2tensor": "07_vision.core.ipynb",
         "PILBase": "07_vision.core.ipynb",
         "PILImage": "07_vision.core.ipynb",
         "PILImageBW": "07_vision.core.ipynb",
         "PILMask": "07_vision.core.ipynb",
         "OpenMask": "07_vision.core.ipynb",
         "OpenMask.loss_func": "07_vision.core.ipynb",
         "PILMask.create": "07_vision.core.ipynb",
         "AddMaskCodes": "07_vision.core.ipynb",
         "TensorPoint": "07_vision.core.ipynb",
         "TensorPointCreate": "07_vision.core.ipynb",
         "TensorPointCreate.loss_func": "07_vision.core.ipynb",
         "TensorPoint.create": "07_vision.core.ipynb",
         "get_annotations": "07_vision.core.ipynb",
         "TensorBBox": "07_vision.core.ipynb",
         "LabeledBBox": "07_vision.core.ipynb",
         "encodes": "40_tabular.core.ipynb",
         "PointScaler": "07_vision.core.ipynb",
         "BBoxLabeler": "07_vision.core.ipynb",
         "decodes": "40_tabular.core.ipynb",
         "get_grid": "08_vision.data.ipynb",
         "clip_remove_empty": "08_vision.data.ipynb",
         "bb_pad": "08_vision.data.ipynb",
         "ImageBlock": "08_vision.data.ipynb",
         "MaskBlock": "08_vision.data.ipynb",
         "PointBlock": "08_vision.data.ipynb",
         "BBoxBlock": "08_vision.data.ipynb",
         "PointBlock.__doc__": "08_vision.data.ipynb",
         "BBoxBlock.__doc__": "08_vision.data.ipynb",
         "BBoxLblBlock": "08_vision.data.ipynb",
         "ImageDataLoaders": "08_vision.data.ipynb",
         "ImageDataLoaders.from_csv": "08_vision.data.ipynb",
         "ImageDataLoaders.from_name_func": "08_vision.data.ipynb",
         "ImageDataLoaders.from_path_re": "08_vision.data.ipynb",
         "ImageDataLoaders.from_name_re": "08_vision.data.ipynb",
         "SegmentationDataLoaders": "08_vision.data.ipynb",
         "RandTransform": "09_vision.augment.ipynb",
         "TensorTypes": "09_vision.augment.ipynb",
         "Image.Image.flip_lr": "09_vision.augment.ipynb",
         "TensorImageBase.flip_lr": "09_vision.augment.ipynb",
         "TensorPoint.flip_lr": "09_vision.augment.ipynb",
         "TensorBBox.flip_lr": "09_vision.augment.ipynb",
         "FlipItem": "09_vision.augment.ipynb",
         "PILImage.dihedral": "09_vision.augment.ipynb",
         "TensorImage.dihedral": "09_vision.augment.ipynb",
         "TensorPoint.dihedral": "09_vision.augment.ipynb",
         "TensorBBox.dihedral": "09_vision.augment.ipynb",
         "DihedralItem": "09_vision.augment.ipynb",
         "TensorBBox.crop_pad": "09_vision.augment.ipynb",
         "TensorPoint.crop_pad": "09_vision.augment.ipynb",
         "Image.Image.crop_pad": "09_vision.augment.ipynb",
         "CropPad": "09_vision.augment.ipynb",
         "RandomCrop": "09_vision.augment.ipynb",
         "OldRandomCrop": "09_vision.augment.ipynb",
         "Resize": "09_vision.augment.ipynb",
         "RandomResizedCrop": "09_vision.augment.ipynb",
         "RatioResize": "09_vision.augment.ipynb",
         "TensorImage.affine_coord": "09_vision.augment.ipynb",
         "TensorMask.affine_coord": "09_vision.augment.ipynb",
         "TensorPoint.affine_coord": "09_vision.augment.ipynb",
         "TensorBBox.affine_coord": "09_vision.augment.ipynb",
         "AffineCoordTfm": "09_vision.augment.ipynb",
         "RandomResizedCropGPU": "09_vision.augment.ipynb",
         "affine_mat": "09_vision.augment.ipynb",
         "mask_tensor": "09_vision.augment.ipynb",
         "flip_mat": "09_vision.augment.ipynb",
         "TensorImage.flip_batch": "09_vision.augment.ipynb",
         "TensorMask.flip_batch": "09_vision.augment.ipynb",
         "TensorPoint.flip_batch": "09_vision.augment.ipynb",
         "TensorBBox.flip_batch": "09_vision.augment.ipynb",
         "Flip": "09_vision.augment.ipynb",
         "DeterministicDraw": "09_vision.augment.ipynb",
         "DeterministicFlip": "09_vision.augment.ipynb",
         "dihedral_mat": "09_vision.augment.ipynb",
         "TensorImage.dihedral_batch": "09_vision.augment.ipynb",
         "TensorMask.dihedral_batch": "09_vision.augment.ipynb",
         "TensorPoint.dihedral_batch": "09_vision.augment.ipynb",
         "TensorBBox.dihedral_batch": "09_vision.augment.ipynb",
         "Dihedral": "09_vision.augment.ipynb",
         "DeterministicDihedral": "09_vision.augment.ipynb",
         "rotate_mat": "09_vision.augment.ipynb",
         "TensorImage.rotate": "09_vision.augment.ipynb",
         "TensorMask.rotate": "09_vision.augment.ipynb",
         "TensorPoint.rotate": "09_vision.augment.ipynb",
         "TensorBBox.rotate": "09_vision.augment.ipynb",
         "Rotate": "09_vision.augment.ipynb",
         "zoom_mat": "09_vision.augment.ipynb",
         "TensorImage.zoom": "09_vision.augment.ipynb",
         "TensorMask.zoom": "09_vision.augment.ipynb",
         "TensorPoint.zoom": "09_vision.augment.ipynb",
         "TensorBBox.zoom": "09_vision.augment.ipynb",
         "Zoom": "09_vision.augment.ipynb",
         "find_coeffs": "09_vision.augment.ipynb",
         "apply_perspective": "09_vision.augment.ipynb",
         "TensorImage.warp": "09_vision.augment.ipynb",
         "TensorMask.warp": "09_vision.augment.ipynb",
         "TensorPoint.warp": "09_vision.augment.ipynb",
         "TensorBBox.warp": "09_vision.augment.ipynb",
         "Warp": "09_vision.augment.ipynb",
         "TensorImage.lighting": "09_vision.augment.ipynb",
         "LightingTfm": "09_vision.augment.ipynb",
         "TensorImage.brightness": "09_vision.augment.ipynb",
         "Brightness": "09_vision.augment.ipynb",
         "TensorImage.contrast": "09_vision.augment.ipynb",
         "Contrast": "09_vision.augment.ipynb",
         "grayscale": "09_vision.augment.ipynb",
         "TensorImage.saturation": "09_vision.augment.ipynb",
         "Saturation": "09_vision.augment.ipynb",
         "rgb2hsv": "09_vision.augment.ipynb",
         "hsv2rgb": "09_vision.augment.ipynb",
         "TensorImage.hue": "09_vision.augment.ipynb",
         "Hue": "09_vision.augment.ipynb",
         "cutout_gaussian": "09_vision.augment.ipynb",
         "norm_apply_denorm": "09_vision.augment.ipynb",
         "RandomErasing": "09_vision.augment.ipynb",
         "setup_aug_tfms": "09_vision.augment.ipynb",
         "aug_transforms": "09_vision.augment.ipynb",
         "download_images": "09b_vision.utils.ipynb",
         "resize_to": "09b_vision.utils.ipynb",
         "verify_image": "09b_vision.utils.ipynb",
         "verify_images": "09b_vision.utils.ipynb",
         "resize_image": "09b_vision.utils.ipynb",
         "resize_images": "09b_vision.utils.ipynb",
         "Box.__getitem__": "09c_vision.widgets.ipynb",
         "widget": "09c_vision.widgets.ipynb",
         "carousel": "09c_vision.widgets.ipynb",
         "ImagesCleaner": "09c_vision.widgets.ipynb",
         "ImageClassifierCleaner": "09c_vision.widgets.ipynb",
         "init_cnn": "11_vision.models.xresnet.ipynb",
         "XResNet": "11_vision.models.xresnet.ipynb",
         "xresnet18": "11_vision.models.xresnet.ipynb",
         "xresnet34": "11_vision.models.xresnet.ipynb",
         "xresnet50": "11_vision.models.xresnet.ipynb",
         "xresnet101": "11_vision.models.xresnet.ipynb",
         "xresnet152": "11_vision.models.xresnet.ipynb",
         "xresnet18_deep": "11_vision.models.xresnet.ipynb",
         "xresnet34_deep": "11_vision.models.xresnet.ipynb",
         "xresnet50_deep": "11_vision.models.xresnet.ipynb",
         "xresnet18_deeper": "11_vision.models.xresnet.ipynb",
         "xresnet34_deeper": "11_vision.models.xresnet.ipynb",
         "xresnet50_deeper": "11_vision.models.xresnet.ipynb",
         "se_kwargs1": "11_vision.models.xresnet.ipynb",
         "se_kwargs2": "11_vision.models.xresnet.ipynb",
         "se_kwargs3": "11_vision.models.xresnet.ipynb",
         "g0": "11_vision.models.xresnet.ipynb",
         "g1": "11_vision.models.xresnet.ipynb",
         "g2": "11_vision.models.xresnet.ipynb",
         "g3": "11_vision.models.xresnet.ipynb",
         "xse_resnet18": "11_vision.models.xresnet.ipynb",
         "xse_resnext18": "11_vision.models.xresnet.ipynb",
         "xresnext18": "11_vision.models.xresnet.ipynb",
         "xse_resnet34": "11_vision.models.xresnet.ipynb",
         "xse_resnext34": "11_vision.models.xresnet.ipynb",
         "xresnext34": "11_vision.models.xresnet.ipynb",
         "xse_resnet50": "11_vision.models.xresnet.ipynb",
         "xse_resnext50": "11_vision.models.xresnet.ipynb",
         "xresnext50": "11_vision.models.xresnet.ipynb",
         "xse_resnet101": "11_vision.models.xresnet.ipynb",
         "xse_resnext101": "11_vision.models.xresnet.ipynb",
         "xresnext101": "11_vision.models.xresnet.ipynb",
         "xse_resnet152": "11_vision.models.xresnet.ipynb",
         "xsenet154": "11_vision.models.xresnet.ipynb",
         "xse_resnext18_deep": "11_vision.models.xresnet.ipynb",
         "xse_resnext34_deep": "11_vision.models.xresnet.ipynb",
         "xse_resnext50_deep": "11_vision.models.xresnet.ipynb",
         "xse_resnext18_deeper": "11_vision.models.xresnet.ipynb",
         "xse_resnext34_deeper": "11_vision.models.xresnet.ipynb",
         "xse_resnext50_deeper": "11_vision.models.xresnet.ipynb",
         "Optimizer": "12_optimizer.ipynb",
         "sgd_step": "12_optimizer.ipynb",
         "weight_decay": "12_optimizer.ipynb",
         "weight_decay.defaults": "12_optimizer.ipynb",
         "l2_reg": "12_optimizer.ipynb",
         "l2_reg.defaults": "12_optimizer.ipynb",
         "average_grad": "12_optimizer.ipynb",
         "average_grad.defaults": "12_optimizer.ipynb",
         "average_sqr_grad": "12_optimizer.ipynb",
         "average_sqr_grad.defaults": "12_optimizer.ipynb",
         "momentum_step": "12_optimizer.ipynb",
         "SGD": "12_optimizer.ipynb",
         "rms_prop_step": "12_optimizer.ipynb",
         "rms_prop_step.defaults": "12_optimizer.ipynb",
         "RMSProp": "12_optimizer.ipynb",
         "step_stat": "12_optimizer.ipynb",
         "debias": "12_optimizer.ipynb",
         "adam_step": "12_optimizer.ipynb",
         "Adam": "12_optimizer.ipynb",
         "radam_step": "12_optimizer.ipynb",
         "RAdam": "12_optimizer.ipynb",
         "qhadam_step": "12_optimizer.ipynb",
         "QHAdam": "12_optimizer.ipynb",
         "larc_layer_lr": "12_optimizer.ipynb",
         "larc_layer_lr.defaults": "12_optimizer.ipynb",
         "larc_step": "12_optimizer.ipynb",
         "Larc": "12_optimizer.ipynb",
         "lamb_step": "12_optimizer.ipynb",
         "Lamb": "12_optimizer.ipynb",
         "Lookahead": "12_optimizer.ipynb",
         "ranger": "12_optimizer.ipynb",
         "detuplify_pg": "12_optimizer.ipynb",
         "set_item_pg": "12_optimizer.ipynb",
         "pytorch_hp_map": "12_optimizer.ipynb",
         "OptimWrapper": "12_optimizer.ipynb",
         "Callback": "13_callback.core.ipynb",
         "TrainEvalCallback": "13_callback.core.ipynb",
         "GatherPredsCallback": "13_callback.core.ipynb",
         "FetchPredsCallback": "13_callback.core.ipynb",
         "defaults.lr": "13a_learner.ipynb",
         "replacing_yield": "13a_learner.ipynb",
         "mk_metric": "13a_learner.ipynb",
         "save_model": "13a_learner.ipynb",
         "load_model": "13a_learner.ipynb",
         "Learner": "13a_learner.ipynb",
         "before_batch_cb": "13a_learner.ipynb",
         "to_detach_from_dl": "13a_learner.ipynb",
         "Metric": "13a_learner.ipynb",
         "AvgMetric": "13a_learner.ipynb",
         "AvgLoss": "13a_learner.ipynb",
         "AvgSmoothLoss": "13a_learner.ipynb",
         "ValueMetric": "13a_learner.ipynb",
         "Recorder": "13a_learner.ipynb",
         "Learner.freeze_to": "13a_learner.ipynb",
         "Learner.freeze": "13a_learner.ipynb",
         "Learner.unfreeze": "13a_learner.ipynb",
         "Learner.export": "13a_learner.ipynb",
         "load_learner": "13a_learner.ipynb",
         "Learner.tta": "13a_learner.ipynb",
         "Learner.gather_args": "13a_learner.ipynb",
         "flatten_check": "13b_metrics.ipynb",
         "AccumMetric": "13b_metrics.ipynb",
         "skm_to_fastai": "13b_metrics.ipynb",
         "optim_metric": "13b_metrics.ipynb",
         "accuracy": "13b_metrics.ipynb",
         "error_rate": "13b_metrics.ipynb",
         "top_k_accuracy": "13b_metrics.ipynb",
         "APScoreBinary": "13b_metrics.ipynb",
         "BalancedAccuracy": "13b_metrics.ipynb",
         "BrierScore": "13b_metrics.ipynb",
         "CohenKappa": "13b_metrics.ipynb",
         "F1Score": "13b_metrics.ipynb",
         "FBeta": "13b_metrics.ipynb",
         "HammingLoss": "13b_metrics.ipynb",
         "Jaccard": "13b_metrics.ipynb",
         "Precision": "13b_metrics.ipynb",
         "Recall": "13b_metrics.ipynb",
         "RocAuc": "13b_metrics.ipynb",
         "RocAucBinary": "13b_metrics.ipynb",
         "MatthewsCorrCoef": "13b_metrics.ipynb",
         "Perplexity": "13b_metrics.ipynb",
         "perplexity": "13b_metrics.ipynb",
         "accuracy_multi": "13b_metrics.ipynb",
         "APScoreMulti": "13b_metrics.ipynb",
         "BrierScoreMulti": "13b_metrics.ipynb",
         "F1ScoreMulti": "13b_metrics.ipynb",
         "FBetaMulti": "13b_metrics.ipynb",
         "HammingLossMulti": "13b_metrics.ipynb",
         "JaccardMulti": "13b_metrics.ipynb",
         "MatthewsCorrCoefMulti": "13b_metrics.ipynb",
         "PrecisionMulti": "13b_metrics.ipynb",
         "RecallMulti": "13b_metrics.ipynb",
         "RocAucMulti": "13b_metrics.ipynb",
         "mse": "13b_metrics.ipynb",
         "rmse": "13b_metrics.ipynb",
         "rmse.__doc__": "13b_metrics.ipynb",
         "mae": "13b_metrics.ipynb",
         "msle": "13b_metrics.ipynb",
         "exp_rmspe": "13b_metrics.ipynb",
         "exp_rmspe.__doc__": "13b_metrics.ipynb",
         "ExplainedVariance": "13b_metrics.ipynb",
         "R2Score": "13b_metrics.ipynb",
         "PearsonCorrCoef": "13b_metrics.ipynb",
         "SpearmanCorrCoef": "13b_metrics.ipynb",
         "foreground_acc": "13b_metrics.ipynb",
         "Dice": "13b_metrics.ipynb",
         "JaccardCoeff": "13b_metrics.ipynb",
         "CorpusBLEUMetric": "13b_metrics.ipynb",
         "LossMetric": "13b_metrics.ipynb",
         "LossMetrics": "13b_metrics.ipynb",
         "annealer": "14_callback.schedule.ipynb",
         "sched_lin": "14_callback.schedule.ipynb",
         "sched_cos": "14_callback.schedule.ipynb",
         "sched_no": "14_callback.schedule.ipynb",
         "sched_exp": "14_callback.schedule.ipynb",
         "SchedLin": "14_callback.schedule.ipynb",
         "SchedCos": "14_callback.schedule.ipynb",
         "SchedNo": "14_callback.schedule.ipynb",
         "SchedExp": "14_callback.schedule.ipynb",
         "SchedLin.__doc__": "14_callback.schedule.ipynb",
         "SchedCos.__doc__": "14_callback.schedule.ipynb",
         "SchedExp.__doc__": "14_callback.schedule.ipynb",
         "SchedPoly": "14_callback.schedule.ipynb",
         "combine_scheds": "14_callback.schedule.ipynb",
         "combined_cos": "14_callback.schedule.ipynb",
         "ParamScheduler": "14_callback.schedule.ipynb",
         "Learner.fit_one_cycle": "14_callback.schedule.ipynb",
         "Recorder.plot_sched": "14_callback.schedule.ipynb",
         "Learner.fit_flat_cos": "14_callback.schedule.ipynb",
         "Learner.fit_sgdr": "14_callback.schedule.ipynb",
         "Learner.fine_tune": "14_callback.schedule.ipynb",
         "LRFinder": "14_callback.schedule.ipynb",
         "Recorder.plot_lr_find": "14_callback.schedule.ipynb",
         "SuggestedLRs": "14_callback.schedule.ipynb",
         "Learner.lr_find": "14_callback.schedule.ipynb",
         "CollectDataCallback": "14a_callback.data.ipynb",
         "CudaCallback": "14a_callback.data.ipynb",
         "WeightedDL": "14a_callback.data.ipynb",
         "Datasets.weighted_dataloaders": "14a_callback.data.ipynb",
         "PartialDL": "14a_callback.data.ipynb",
         "FilteredBase.partial_dataloaders": "14a_callback.data.ipynb",
         "Hook": "15_callback.hook.ipynb",
         "hook_output": "15_callback.hook.ipynb",
         "Hooks": "15_callback.hook.ipynb",
         "hook_outputs": "15_callback.hook.ipynb",
         "dummy_eval": "15_callback.hook.ipynb",
         "model_sizes": "15_callback.hook.ipynb",
         "num_features_model": "15_callback.hook.ipynb",
         "has_params": "15_callback.hook.ipynb",
         "HookCallback": "15_callback.hook.ipynb",
         "total_params": "15_callback.hook.ipynb",
         "layer_info": "15_callback.hook.ipynb",
         "module_summary": "15_callback.hook.ipynb",
         "Learner.summary": "15_callback.hook.ipynb",
         "ActivationStats": "15_callback.hook.ipynb",
         "UnetBlock": "15a_vision.models.unet.ipynb",
         "ResizeToOrig": "15a_vision.models.unet.ipynb",
         "DynamicUnet": "15a_vision.models.unet.ipynb",
         "ProgressCallback": "16_callback.progress.ipynb",
         "Learner.no_bar": "16_callback.progress.ipynb",
         "ShowGraphCallback": "16_callback.progress.ipynb",
         "CSVLogger": "16_callback.progress.ipynb",
         "TerminateOnNaNCallback": "17_callback.tracker.ipynb",
         "TrackerCallback": "17_callback.tracker.ipynb",
         "EarlyStoppingCallback": "17_callback.tracker.ipynb",
         "SaveModelCallback": "17_callback.tracker.ipynb",
         "ReduceLROnPlateau": "17_callback.tracker.ipynb",
         "get_master": "18_callback.fp16.ipynb",
         "to_master_grads": "18_callback.fp16.ipynb",
         "to_model_params": "18_callback.fp16.ipynb",
         "test_overflow": "18_callback.fp16.ipynb",
         "grad_overflow": "18_callback.fp16.ipynb",
         "copy_clone": "18_callback.fp16.ipynb",
         "ModelToHalf": "18_callback.fp16.ipynb",
         "MixedPrecision": "18_callback.fp16.ipynb",
         "Learner.to_fp16": "18_callback.fp16.ipynb",
         "Learner.to_fp32": "18_callback.fp16.ipynb",
         "NativeMixedPrecision": "18_callback.fp16.ipynb",
         "Learner.to_native_fp16": "18_callback.fp16.ipynb",
         "Learner.to_native_fp32": "18_callback.fp16.ipynb",
         "ShortEpochCallback": "18a_callback.training.ipynb",
         "GradientAccumulation": "18a_callback.training.ipynb",
         "set_bn_eval": "18a_callback.training.ipynb",
         "BnFreeze": "18a_callback.training.ipynb",
         "bn_types": "18a_callback.training.ipynb",
         "MCDropoutCallback": "18b_callback.preds.ipynb",
         "reduce_loss": "19_callback.mixup.ipynb",
         "MixUp": "19_callback.mixup.ipynb",
         "Interpretation": "20_interpret.ipynb",
         "ClassificationInterpretation": "20_interpret.ipynb",
         "DataParallel.reset": "20a_distributed.ipynb",
         "ParallelTrainer": "20a_distributed.ipynb",
         "Learner.to_parallel": "20a_distributed.ipynb",
         "Learner.detach_parallel": "20a_distributed.ipynb",
         "Learner.parallel_ctx": "20a_distributed.ipynb",
         "DistributedDataParallel.reset": "20a_distributed.ipynb",
         "setup_distrib": "20a_distributed.ipynb",
         "teardown_distrib": "20a_distributed.ipynb",
         "DistributedDL": "20a_distributed.ipynb",
         "DistributedTrainer": "20a_distributed.ipynb",
         "Learner.to_distributed": "20a_distributed.ipynb",
         "Learner.detach_distributed": "20a_distributed.ipynb",
         "Learner.distrib_ctx": "20a_distributed.ipynb",
         "rank0_first": "20a_distributed.ipynb",
         "has_pool_type": "21_vision.learner.ipynb",
         "create_body": "21_vision.learner.ipynb",
         "create_head": "21_vision.learner.ipynb",
         "create_cnn_model": "21_vision.learner.ipynb",
         "cnn_config": "21_vision.learner.ipynb",
         "default_split": "21_vision.learner.ipynb",
         "model_meta": "21_vision.learner.ipynb",
         "cnn_learner": "21_vision.learner.ipynb",
         "unet_config": "21_vision.learner.ipynb",
         "unet_learner": "21_vision.learner.ipynb",
         "GANModule": "24_vision.gan.ipynb",
         "basic_critic": "24_vision.gan.ipynb",
         "AddChannels": "24_vision.gan.ipynb",
         "basic_generator": "24_vision.gan.ipynb",
         "DenseResBlock": "24_vision.gan.ipynb",
         "gan_critic": "24_vision.gan.ipynb",
         "GANLoss": "24_vision.gan.ipynb",
         "AdaptiveLoss": "24_vision.gan.ipynb",
         "accuracy_thresh_expand": "24_vision.gan.ipynb",
         "set_freeze_model": "24_vision.gan.ipynb",
         "GANTrainer": "24_vision.gan.ipynb",
         "FixedGANSwitcher": "24_vision.gan.ipynb",
         "AdaptiveGANSwitcher": "24_vision.gan.ipynb",
         "GANDiscriminativeLR": "24_vision.gan.ipynb",
         "InvisibleTensor": "24_vision.gan.ipynb",
         "generate_noise": "24_vision.gan.ipynb",
         "gan_loss_from_func": "24_vision.gan.ipynb",
         "GANLearner": "24_vision.gan.ipynb",
         "GANLearner.from_learners": "24_vision.gan.ipynb",
         "GANLearner.wgan": "24_vision.gan.ipynb",
         "spec_add_spaces": "30_text.core.ipynb",
         "rm_useless_spaces": "30_text.core.ipynb",
         "replace_rep": "30_text.core.ipynb",
         "replace_wrep": "30_text.core.ipynb",
         "fix_html": "30_text.core.ipynb",
         "replace_all_caps": "30_text.core.ipynb",
         "replace_maj": "30_text.core.ipynb",
         "lowercase": "30_text.core.ipynb",
         "replace_space": "30_text.core.ipynb",
         "defaults.text_spec_tok": "30_text.core.ipynb",
         "defaults.text_proc_rules": "30_text.core.ipynb",
         "defaults.text_postproc_rules": "30_text.core.ipynb",
         "BaseTokenizer": "30_text.core.ipynb",
         "SpacyTokenizer": "30_text.core.ipynb",
         "WordTokenizer": "30_text.core.ipynb",
         "TokenizeWithRules": "30_text.core.ipynb",
         "tokenize1": "30_text.core.ipynb",
         "parallel_tokenize": "30_text.core.ipynb",
         "fn_counter_pkl": "30_text.core.ipynb",
         "fn_lengths_pkl": "30_text.core.ipynb",
         "tokenize_folder": "30_text.core.ipynb",
         "tokenize_files": "30_text.core.ipynb",
         "tokenize_texts": "30_text.core.ipynb",
         "tokenize_df": "30_text.core.ipynb",
         "tokenize_csv": "30_text.core.ipynb",
         "load_tokenized_csv": "30_text.core.ipynb",
         "Tokenizer": "30_text.core.ipynb",
         "eu_langs": "30_text.core.ipynb",
         "SentencePieceTokenizer": "30_text.core.ipynb",
         "SubwordTokenizer": "30_text.core.ipynb",
         "reverse_text": "31_text.data.ipynb",
         "make_vocab": "31_text.data.ipynb",
         "TensorText": "31_text.data.ipynb",
         "LMTensorText": "31_text.data.ipynb",
         "TensorText.__doc__": "31_text.data.ipynb",
         "LMTensorText.__doc__": "31_text.data.ipynb",
         "Numericalize": "31_text.data.ipynb",
         "LMDataLoader": "31_text.data.ipynb",
         "pad_input": "31_text.data.ipynb",
         "pad_input_chunk": "31_text.data.ipynb",
         "SortedDL": "31_text.data.ipynb",
         "TextBlock": "31_text.data.ipynb",
         "TextDataLoaders": "31_text.data.ipynb",
         "TextDataLoaders.from_csv": "31_text.data.ipynb",
         "dropout_mask": "32_text.models.awdlstm.ipynb",
         "RNNDropout": "32_text.models.awdlstm.ipynb",
         "WeightDropout": "32_text.models.awdlstm.ipynb",
         "EmbeddingDropout": "32_text.models.awdlstm.ipynb",
         "AWD_LSTM": "32_text.models.awdlstm.ipynb",
         "awd_lstm_lm_split": "32_text.models.awdlstm.ipynb",
         "awd_lstm_lm_config": "32_text.models.awdlstm.ipynb",
         "awd_lstm_clas_split": "32_text.models.awdlstm.ipynb",
         "awd_lstm_clas_config": "32_text.models.awdlstm.ipynb",
         "AWD_QRNN": "32_text.models.awdlstm.ipynb",
         "awd_qrnn_lm_config": "32_text.models.awdlstm.ipynb",
         "awd_qrnn_clas_config": "32_text.models.awdlstm.ipynb",
         "LinearDecoder": "33_text.models.core.ipynb",
         "SequentialRNN": "33_text.models.core.ipynb",
         "get_language_model": "33_text.models.core.ipynb",
         "SentenceEncoder": "33_text.models.core.ipynb",
         "masked_concat_pool": "33_text.models.core.ipynb",
         "PoolingLinearClassifier": "33_text.models.core.ipynb",
         "get_text_classifier": "33_text.models.core.ipynb",
         "ModelResetter": "34_callback.rnn.ipynb",
         "RNNRegularizer": "34_callback.rnn.ipynb",
         "load_cpp": "36_text.models.qrnn.ipynb",
         "forget_mult_cuda": "36_text.models.qrnn.ipynb",
         "bwd_forget_mult_cuda": "36_text.models.qrnn.ipynb",
         "dispatch_cuda": "36_text.models.qrnn.ipynb",
         "forget_mult_CPU": "36_text.models.qrnn.ipynb",
         "ForgetMultGPU": "36_text.models.qrnn.ipynb",
         "QRNNLayer": "36_text.models.qrnn.ipynb",
         "QRNN": "36_text.models.qrnn.ipynb",
         "match_embeds": "37_text.learner.ipynb",
         "load_ignore_keys": "37_text.learner.ipynb",
         "clean_raw_keys": "37_text.learner.ipynb",
         "load_model_text": "37_text.learner.ipynb",
         "TextLearner": "37_text.learner.ipynb",
         "decode_spec_tokens": "37_text.learner.ipynb",
         "LMLearner": "37_text.learner.ipynb",
         "language_model_learner": "37_text.learner.ipynb",
         "text_classifier_learner": "37_text.learner.ipynb",
         "make_date": "40_tabular.core.ipynb",
         "add_datepart": "40_tabular.core.ipynb",
         "add_elapsed_times": "40_tabular.core.ipynb",
         "cont_cat_split": "40_tabular.core.ipynb",
         "df_shrink_dtypes": "40_tabular.core.ipynb",
         "df_shrink": "40_tabular.core.ipynb",
         "Tabular": "40_tabular.core.ipynb",
         "TabularPandas": "40_tabular.core.ipynb",
         "TabularProc": "40_tabular.core.ipynb",
         "Categorify": "40_tabular.core.ipynb",
         "setups": "40_tabular.core.ipynb",
         "FillStrategy": "40_tabular.core.ipynb",
         "FillMissing": "40_tabular.core.ipynb",
         "ReadTabBatch": "40_tabular.core.ipynb",
         "TabDataLoader": "40_tabular.core.ipynb",
         "TabularDataLoaders": "41_tabular.data.ipynb",
         "TabularDataLoaders.from_csv": "41_tabular.data.ipynb",
         "emb_sz_rule": "42_tabular.model.ipynb",
         "get_emb_sz": "42_tabular.model.ipynb",
         "TabularModel": "42_tabular.model.ipynb",
         "tabular_config": "42_tabular.model.ipynb",
         "TabularLearner": "43_tabular.learner.ipynb",
         "tabular_learner": "43_tabular.learner.ipynb",
         "TabularCollab": "45_collab.ipynb",
         "CollabDataLoaders": "45_collab.ipynb",
         "CollabDataLoaders.from_csv": "45_collab.ipynb",
         "EmbeddingDotBias": "45_collab.ipynb",
         "EmbeddingNN": "45_collab.ipynb",
         "collab_learner": "45_collab.ipynb",
         "get_dicom_files": "60_medical.imaging.ipynb",
         "Path.dcmread": "60_medical.imaging.ipynb",
         "TensorDicom": "60_medical.imaging.ipynb",
         "PILDicom": "60_medical.imaging.ipynb",
         "pixels": "60_medical.imaging.ipynb",
         "scaled_px": "60_medical.imaging.ipynb",
         "array_freqhist_bins": "60_medical.imaging.ipynb",
         "Tensor.freqhist_bins": "60_medical.imaging.ipynb",
         "Tensor.hist_scaled_pt": "60_medical.imaging.ipynb",
         "Tensor.hist_scaled": "60_medical.imaging.ipynb",
         "DcmDataset.hist_scaled": "60_medical.imaging.ipynb",
         "Tensor.windowed": "60_medical.imaging.ipynb",
         "DcmDataset.windowed": "60_medical.imaging.ipynb",
         "dicom_windows": "60_medical.imaging.ipynb",
         "TensorCTScan": "60_medical.imaging.ipynb",
         "PILCTScan": "60_medical.imaging.ipynb",
         "DcmDataset.show": "60_medical.imaging.ipynb",
         "DcmDataset.pct_in_window": "60_medical.imaging.ipynb",
         "uniform_blur2d": "60_medical.imaging.ipynb",
         "gauss_blur2d": "60_medical.imaging.ipynb",
         "Tensor.mask_from_blur": "60_medical.imaging.ipynb",
         "DcmDataset.mask_from_blur": "60_medical.imaging.ipynb",
         "mask2bbox": "60_medical.imaging.ipynb",
         "crop_resize": "60_medical.imaging.ipynb",
         "Tensor.to_nchan": "60_medical.imaging.ipynb",
         "DcmDataset.to_nchan": "60_medical.imaging.ipynb",
         "Tensor.to_3chan": "60_medical.imaging.ipynb",
         "DcmDataset.to_3chan": "60_medical.imaging.ipynb",
         "Tensor.save_jpg": "60_medical.imaging.ipynb",
         "DcmDataset.save_jpg": "60_medical.imaging.ipynb",
         "Tensor.to_uint16": "60_medical.imaging.ipynb",
         "DcmDataset.to_uint16": "60_medical.imaging.ipynb",
         "Tensor.save_tif16": "60_medical.imaging.ipynb",
         "DcmDataset.save_tif16": "60_medical.imaging.ipynb",
         "DcmDataset.set_pixels": "60_medical.imaging.ipynb",
         "DcmDataset.pixel_array": "60_medical.imaging.ipynb",
         "DcmDataset.zoom": "60_medical.imaging.ipynb",
         "DcmDataset.zoom_to": "60_medical.imaging.ipynb",
         "DcmDataset.as_dict": "60_medical.imaging.ipynb",
         "pd.DataFrame.from_dicoms": "60_medical.imaging.ipynb",
         "WandbCallback": "70_callback.wandb.ipynb",
         "log_dataset": "70_callback.wandb.ipynb",
         "log_model": "70_callback.wandb.ipynb",
         "TensorBoardBaseCallback": "71_callback.tensorboard.ipynb",
         "TensorBoardCallback": "71_callback.tensorboard.ipynb",
         "TensorBoardProjectorCallback": "71_callback.tensorboard.ipynb",
         "NeptuneCallback": "72_callback.neptune.ipynb",
         "json_clean": "73_callback.captum.ipynb",
         "jsonutil.json_clean": "73_callback.captum.ipynb",
         "CaptumInterpretation": "73_callback.captum.ipynb",
         "CaptumInterpretation.insights": "73_callback.captum.ipynb",
         "CutMix": "74_callback.cutmix.ipynb",
         "synth_dbunch": "97_test_utils.ipynb",
         "RegModel": "97_test_utils.ipynb",
         "synth_learner": "97_test_utils.ipynb",
         "VerboseCallback": "97_test_utils.ipynb",
         "PYTORCH_URL": "99_pytorch_doc.ipynb",
         "pytorch_doc_link": "99_pytorch_doc.ipynb"}

modules = ["torch_core.py",
           "layers.py",
           "losses.py",
           "data/load.py",
           "data/core.py",
           "data/external.py",
           "data/transforms.py",
           "data/block.py",
           "vision/core.py",
           "vision/data.py",
           "vision/augment.py",
           "vision/utils.py",
           "vision/widgets.py",
           "vision/models/xresnet.py",
           "optimizer.py",
           "callback/core.py",
           "learner.py",
           "metrics.py",
           "callback/schedule.py",
           "callback/data.py",
           "callback/hook.py",
           "vision/models/unet.py",
           "callback/progress.py",
           "callback/tracker.py",
           "callback/fp16.py",
           "callback/training.py",
           "callback/preds.py",
           "callback/mixup.py",
           "interpret.py",
           "distributed.py",
           "vision/learner.py",
           "vision/gan.py",
           "text/core.py",
           "text/data.py",
           "text/models/awdlstm.py",
           "text/models/core.py",
           "callback/rnn.py",
           "text/models/qrnn.py",
           "text/learner.py",
           "tabular/core.py",
           "tabular/data.py",
           "tabular/model.py",
           "tabular/learner.py",
           "collab.py",
           "medical/imaging.py",
           "medical/text.py",
           "callback/wandb.py",
           "callback/tensorboard.py",
           "callback/neptune.py",
           "callback/captum.py",
           "callback/cutmix.py",
           "test_utils.py",
           "_pytorch_doc.py"]

doc_url = "https://docs.fast.ai/"

git_url = "https://github.com/fastai/fastai/tree/master/"

def custom_doc_links(name):
    from nbdev.showdoc import try_external_doc_link
    return try_external_doc_link(name, ['fastcore', 'nbdev'])




import subprocess, torch
from fastai.basics import *
from fastscript import *

@call_parse
def main(
    gpus:Param("The GPUs to use for distributed training", str)='all',
    script:Param("Script to run", str, opt=False)='',
    args:Param("Args to pass to script", nargs='...', opt=False)=''
):
    "PyTorch distributed training launch helper that spawns multiple distributed processes"
    # Loosely based on torch.distributed.launch
    current_env = os.environ.copy()
    gpus = list(range(torch.cuda.device_count())) if gpus=='all' else list(gpus)
    current_env["WORLD_SIZE"] = str(len(gpus))
    current_env["MASTER_ADDR"] = '127.0.0.1'
    current_env["MASTER_PORT"] = '29500'

    processes = []
    for i,gpu in enumerate(gpus):
        current_env["RANK"] = str(i)
        cmd = [sys.executable, "-u", script, f"--gpu={gpu}"] + args
        process = subprocess.Popen(cmd, env=current_env)
        processes.append(process)

    for process in processes: process.wait()



# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/74_callback.cutmix.ipynb (unless otherwise specified).

__all__ = ['CutMix']

# Cell
from torch.distributions.beta import Beta
from ..vision.all import *

# Cell
class CutMix(Callback):
    "Implementation of `https://arxiv.org/abs/1905.04899`"
    run_after,run_valid = [Normalize],False
    def __init__(self, alpha=1.): self.distrib = Beta(tensor(alpha), tensor(alpha))
    def before_fit(self):
        self.stack_y = getattr(self.learn.loss_func, 'y_int', False)
        if self.stack_y: self.old_lf,self.learn.loss_func = self.learn.loss_func,self.lf

    def after_fit(self):
        if self.stack_y: self.learn.loss_func = self.old_lf

    def before_batch(self):
        W, H = self.xb[0].size(3), self.xb[0].size(2)
        lam = self.distrib.sample((1,)).squeeze().to(self.x.device)
        lam = torch.stack([lam, 1-lam])
        self.lam = lam.max()
        shuffle = torch.randperm(self.y.size(0)).to(self.x.device)
        xb1,self.yb1 = tuple(L(self.xb).itemgot(shuffle)),tuple(L(self.yb).itemgot(shuffle))
        nx_dims = len(self.x.size())
        x1, y1, x2, y2 = self.rand_bbox(W, H, self.lam)
        self.learn.xb[0][:, :, x1:x2, y1:y2] = xb1[0][:, :, x1:x2, y1:y2]
        self.lam = (1 - ((x2-x1)*(y2-y1))/float(W*H)).item()

        if not self.stack_y:
            ny_dims = len(self.y.size())
            self.learn.yb = tuple(L(self.yb1,self.yb).map_zip(torch.lerp,weight=unsqueeze(self.lam, n=ny_dims-1)))

    def lf(self, pred, *yb):
        if not self.training: return self.old_lf(pred, *yb)
        with NoneReduce(self.old_lf) as lf:
            loss = torch.lerp(lf(pred,*self.yb1), lf(pred,*yb), self.lam)
        return reduce_loss(loss, getattr(self.old_lf, 'reduction', 'mean'))

    def rand_bbox(self, W, H, lam):
        cut_rat = torch.sqrt(1. - lam)
        cut_w = (W * cut_rat).type(torch.long)
        cut_h = (H * cut_rat).type(torch.long)
        # uniform
        cx = torch.randint(0, W, (1,)).to(self.x.device)
        cy = torch.randint(0, H, (1,)).to(self.x.device)
        x1 = torch.clamp(cx - cut_w // 2, 0, W)
        y1 = torch.clamp(cy - cut_h // 2, 0, H)
        x2 = torch.clamp(cx + cut_w // 2, 0, W)
        y2 = torch.clamp(cy + cut_h // 2, 0, H)
        return x1, y1, x2, y2